import { Plugin } from 'ckeditor5/src/core';
import { toWidget, toWidgetEditable, Widget } from 'ckeditor5/src/widget';
import InsertAccordionCommand from './insertaccordioncommand';
import InsertAccordionRowCommand from './insertaccordionrowcommand';
import DeleteAccordionRowCommand from './deleteaccordionrowcommand';
import OpenAccordionPropertiesCommand from './openaccordionpropertiescommand';
import AccordionPropertiesDialog from './accordionpropertiesdialog';

// cSpell:ignore accordion insertsimpleboxcommand

/**
 * CKEditor 5 plugins do not work directly with the DOM. They are defined as
 * plugin-specific data models that are then converted to HTML that
 * is inserted in the DOM.
 *
 * CKEditor 5 internally interacts with accordion as this model:
 * <accordion>
 *    <accordionTitle></accordionTitle>
 *    <accordionContent></accordionContent>
 * </accordion>
 *
 * This file has the logic for defining the accordion model, and for how it is
 * converted to standard DOM markup.
 */
class AccordionEditing extends Plugin {
  /**
   * @inheritDoc
   */
  static get pluginName() {
    return 'AccordionEditing';
  }

  static get requires() {
    return [Widget, AccordionPropertiesDialog];
  }

  init() {
    this._defineSchema();
    this._defineConverters();
    this.editor.commands.add(
      'insertAccordion',
      new InsertAccordionCommand(this.editor),
    );

    this.editor.commands.add(
      'insertAccordionRowAbove',
      new InsertAccordionRowCommand(this.editor, { order: 'above' }),
    );
    this.editor.commands.add(
      'insertAccordionRowBelow',
      new InsertAccordionRowCommand(this.editor, { order: 'below' } ),
    );
    this.editor.commands.add(
      'deleteAccordionRow',
      new DeleteAccordionRowCommand(this.editor, {} ),
    );
    this.editor.commands.add(
      'openAccordionProperties',
      new OpenAccordionPropertiesCommand(this.editor, {} ),
    );
  }

  /*
   * This registers the structure that will be seen by CKEditor 5 as
   * <accordion>
   *    <accordionTitle></accordionTitle>
   *    <accordionContent></accordionContent>
   * </accordion>
   *
   * The logic in _defineConverters() will determine how this is converted to
   * markup.
   */
  _defineSchema() {
    // Schemas are registered via the central `editor` object.
    const { schema } = this.editor.model;

    schema.register('accordion', {
      // Behaves like a self-contained object (e.g. an image).
      isObject: true,
      // Allow in places where other blocks are allowed (e.g. directly in the root).
      allowWhere: '$block',
    });

    schema.register( 'accordionRow', {
      allowIn: 'accordion',
      isLimit: true,
    } );

    schema.register('accordionTitle', {
      isLimit: true,
      allowIn: 'accordion',
      allowContentOf: '$block',
    });

    schema.register('accordionContent', {
      isLimit: true,
      allowIn: 'accordion',
      allowContentOf: '$root',
    });

    schema.addChildCheck((context, childDefinition) => {
      // Disallow accordion inside accordionContent.
      if (
        (context.endsWith('accordionContent') || context.endsWith('accordionTitle')) &&
        childDefinition.name === 'accordion'
      ) {
        return false;
      }
    });
  }

  /**
   * Converters determine how CKEditor 5 models are converted into markup and
   * vice-versa.
   */
  _defineConverters() {
    // Converters are registered via the central editor object.
    const { conversion } = this.editor;

    // Consolidated Upcast Converters: which moves things from the view to the model.
    conversion.for('upcast').add(dispatcher => {
      // Using elementToElement for 'div' with class 'usa-accordion'
      conversion.for('upcast').elementToElement({
        view: {
          name: 'div',
          classes: 'usa-accordion'
        },
        model: (viewElement, conversionApi) => {
          const writer = conversionApi.writer;
          const modelElement = writer.createElement('accordion');

          const elementClass = viewElement.getAttribute('class');
          const dataAllowMultiple = viewElement.getAttribute('data-allow-multiple');

          // this is the old way multiselect was done
          const ariaMultiselectable = viewElement.getAttribute('aria-multiselectable');

          if (elementClass) {
            writer.setAttribute('class', elementClass, modelElement);
          }
          if (dataAllowMultiple || ariaMultiselectable) {
            writer.setAttribute('data-allow-multiple', dataAllowMultiple || ariaMultiselectable, modelElement);
          }

          return modelElement;
        }
      });

      // Convert the h4 element (new accordion title) to the intermediate model element
      conversion.for('upcast').elementToElement({
        view: {
          name: 'h4',
          classes: 'usa-accordion__heading'
        },
        model: 'intermediateAccordionTitle'
      });

      // Convert the p element (old accordion title) to the intermediate model element
      conversion.for('upcast').elementToElement({
        view: {
          name: 'p',
          classes: 'usa-accordion__heading'
        },
        model: 'intermediateAccordionTitle'
      });

      // Convert the button element inside the accordion title
      conversion.for('upcast').elementToElement({
        view: {
          name: 'button',
          classes: 'usa-accordion__button'
        },
        model: (viewElement, conversionApi) => {
          const { writer } = conversionApi;
          const modelElement = writer.createElement('accordionTitle');
          const ariaExpanded = viewElement.getAttribute('aria-expanded');

          // Handle and append raw content inside the button.
          const rawContent = viewElement.getCustomProperty('$rawContent');

          if (rawContent) {
            const tempDiv = document.createElement('div');
            tempDiv.innerHTML = rawContent;
            writer.append(tempDiv.innerText.trim(), modelElement);
          }

          writer.setAttribute('aria-expanded', ariaExpanded === 'true' ? 'true' : 'false', modelElement);

          return modelElement;
        },
        converterPriority: 'high'
      });

      // Handle insertion and additional custom processing if needed
      dispatcher.on('insert:intermediateAccordionTitle', (evt, data, { writer, safeInsert, convertChildren }) => {
        const modelElement = writer.createElement('accordionTitle');

        if (!safeInsert(modelElement, data.modelCursor)) {
          return;
        }

        convertChildren(data.item, modelElement);

        data.result = modelElement;
      });

      dispatcher.on('insert:accordionTitle', (evt, data, conversionApi) => {
        // Optional: Custom handling for accordionTitle model element insertion
        evt.stop();
      });

      conversion.for('upcast').elementToElement({
        model: 'accordionContent',
        view: {
          name: 'div',
          classes: 'usa-accordion__content',
        },
      });
    });

    // Data Downcast Converters: converts stored model data into HTML.
    // These trigger when content is saved.
    conversion.for('dataDowncast').add(dispatcher => {
      conversion.for('dataDowncast').elementToElement({
        model: 'accordion',
        view: (modelElement, { writer }) => {
          const viewElement = writer.createContainerElement('div');

          // Handle custom attributes.
          const elementClass = modelElement.getAttribute('class');
          const dataAllowMultiple = modelElement.getAttribute('data-allow-multiple');

          if (elementClass) {
            writer.setAttribute('class', elementClass, viewElement);
          }

          if (dataAllowMultiple) {
            writer.setAttribute('data-allow-multiple', dataAllowMultiple, viewElement);
          }

          return viewElement;
        }
      });

      conversion.for('dataDowncast').elementToElement({
        model: 'accordionTitle',
        view: (modelElement, { writer, mapper, convertChildren }) => {
          const headingElement = writer.createContainerElement('h4', { class: 'usa-accordion__heading' });
          const button = writer.createEditableElement('button', {
            class: 'usa-accordion__button',
            type: 'button',
            'aria-expanded': modelElement.getAttribute('aria-expanded')
          });

          writer.insert(writer.createPositionAt(headingElement, 0), button);

          mapper.bindElements(modelElement, button);

          convertChildren(modelElement, writer.createPositionAt(button, 0));

          return headingElement;
        }
      });

      /*
      dispatcher.on('attribute:aria-expanded', (evt, data, conversionApi) => {
          if (data.item.name !== 'accordionTitle') return;

          const { writer, mapper } = conversionApi;
          const viewElement = mapper.toViewElement(data.item);

          if (data.attributeNewValue !== undefined) {
              writer.setAttribute('aria-expanded', data.attributeNewValue, viewElement);
          } else {
              writer.removeAttribute('aria-expanded', viewElement);
          }
      });

      dispatcher.on('attribute:class', (evt, data, conversionApi) => {
          if (data.item.name !== 'accordion') return;

          const { writer, mapper } = conversionApi;
          const viewElement = mapper.toViewElement(data.item);

          if (data.attributeNewValue !== undefined) {
              writer.setAttribute('class', data.attributeNewValue, viewElement);
          } else {
              writer.removeAttribute('class', viewElement);
          }
      });

      dispatcher.on('attribute:data-allow-multiple', (evt, data, conversionApi) => {
          if (data.item.name !== 'accordion') return;

          const { writer, mapper } = conversionApi;
          const viewElement = mapper.toViewElement(data.item);

          if (data.attributeNewValue) {
              writer.setAttribute('data-allow-multiple', data.attributeNewValue, viewElement);
          } else {
              writer.removeAttribute('data-allow-multiple', viewElement);
          }
      });
      */

      conversion.for('dataDowncast').elementToElement({
          model: 'accordionContent',
          view: (modelElement, { writer }) =>
              writer.createContainerElement('div', { class: 'usa-accordion__content usa-prose' }),
      });
    });


    // Editing Downcast Converters. These render the content to the user for
    // editing, i.e. this determines what gets seen in the editor. These trigger
    // after the Data Upcast Converters, and are re-triggered any time there
    // are changes to any of the models' properties.
    conversion.for('editingDowncast').add(dispatcher => {
      // Convert the <accordion> model into a container widget in the editor UI.
      conversion.for('editingDowncast').elementToElement({
        model: 'accordion',
        view: (modelElement, { writer: viewWriter }) => {
          const div = viewWriter.createContainerElement('div', {
            class: 'usa-accordion'
          });

          return toWidget(div, viewWriter);
        },
      });

      // Convert the <accordionTitle> model into an editable <div> widget.
      conversion.for('editingDowncast').elementToElement({
        model: 'accordionTitle',
        view: (modelElement, { writer: viewWriter }) => {
          const div = viewWriter.createEditableElement('div', {
            class: 'ckeditor-accordion-title',
          });

          return toWidgetEditable(div, viewWriter);
        },
      });

      // Convert the <accordionContent> model into an editable <div> widget.
      conversion.for('editingDowncast').elementToElement({
        model: 'accordionContent',
        view: (modelElement, { writer: viewWriter }) => {
          const div = viewWriter.createEditableElement('div', {
            class: 'ckeditor-accordion-content',
          });

          return toWidgetEditable(div, viewWriter);
        },
      });
    });
  }
}

export default AccordionEditing;
