/**
 * @file defines AccordionPropertiesDialog.
 */

import { Plugin } from 'ckeditor5/src/core';
import { Dialog, View } from 'ckeditor5/src/ui';

/**
 * The AccordionPropertiesDialog plugin provides a dialog
 * for editing accordion properties within CKEditor.
 */
class AccordionPropertiesDialog extends Plugin {
  /**
   * Returns the name of the plugin.
   * @return {String}
   */
  static get pluginName() {
    return 'AccordionPropertiesDialog';
  }

  /**
   * Creates an instance of AccordionPropertiesDialog.
   * @param {Object} editor - The CKEditor instance.
   */
  constructor(editor) {
    super(editor);
    this.dialog = null;  // Instance of the dialog.
    this._internal = null;  // Internal state for the dialog.
  }

  /**
   * Initializes the plugin by modifying the content policy and creating dialog contents.
   */
  init() {
    const editor = this.editor;

    this._createDialogContents(editor);
  }

  /**
   * Opens the dialog with the given accordion element.
   * @param {Object} accordion - The accordion element to edit.
   */
  open(accordion) {
    this._internal.accordion = accordion;
    this.dialog.show(this._internal);
  }

  /**
   * Closes the dialog.
   */
  close() {
    this.dialog.hide();
  }

  /**
   * Saves the state of the form checkboxes and updates the model attributes accordingly.
   */
  _saveFormState() {
    const editor = this.editor;
    const accordion = this._internal.accordion;
    const content = this._internal.content;

    // Extract values from checkboxes
    const showBorderChecked = content.element.querySelector('#showBorderCheckbox').checked;
    const multiSelectChecked = content.element.querySelector('#multiSelectCheckbox').checked;
    const startCollapsedChecked = content.element.querySelector('#startCollapsedCheckbox').checked;

    editor.model.change(writer => {
      if (showBorderChecked) {
        writer.setAttribute('class', 'usa-accordion usa-accordion--bordered', accordion);
      }
      else {
        writer.setAttribute('class', 'usa-accordion', accordion);
      }

      if (multiSelectChecked) {
        writer.setAttribute('data-allow-multiple', 'true', accordion);
      }
      else {
        writer.removeAttribute('data-allow-multiple', accordion);
      }

      // Update aria-expanded attributes based on the startCollapsedChecked state
      const accordionTitles = Array.from(accordion.getChildren()).filter(child =>
        child.name === 'accordionTitle'
      );

      accordionTitles.forEach((title, index) => {
        if (startCollapsedChecked) {
          writer.setAttribute('aria-expanded', 'false', title);
        }
        else {
          writer.setAttribute('aria-expanded', index === 0 ? 'true' : 'false', title);
        }
      });
    });

    // Ensure the view is updated to reflect the model changes
    editor.editing.view.forceRender();
  }

  /**
   * Loads the current state of attributes into the dialog form checkboxes.
   */
  _loadFormState() {
    const accordion = this._internal.accordion;
    const content = this._internal.content;
    const classAttribute = accordion.getAttribute('class') || '';

    content.element.querySelector('#showBorderCheckbox').checked = classAttribute.includes('usa-accordion--bordered');
    content.element.querySelector('#multiSelectCheckbox').checked = accordion.hasAttribute('data-allow-multiple');

    // Check if all accordion titles are collapsed
    const accordionTitles = Array.from(accordion.getChildren()).filter(child => child.name === 'accordionTitle');
    content.element.querySelector('#startCollapsedCheckbox').checked = accordionTitles.every(title => title.getAttribute('aria-expanded') === 'false');
  }

  /**
   * Creates the dialog contents, including the form and buttons.
   * @param {Object} editor - The CKEditor instance.
   */
  _createDialogContents(editor) {
    const t = editor.t;

    // Initialize the dialog
    this.dialog = new Dialog(editor);
    editor.ui.componentFactory.add('accordionProperties', () => this.dialog);

    const content = new View();
    content.setTemplate({
      tag: 'form',
      attributes: {
        style: 'padding:1rem'
      },
      children: [
        {
          tag: 'div',
          children: [
            {
              tag: 'input',
              attributes: {
                id: 'showBorderCheckbox',
                style: 'margin-right:1rem',
                type: 'checkbox',
              }
            },
            {
              tag: 'label',
              attributes: {
                for: 'showBorderCheckbox'
              },
              children: [
                {
                  text: t('Make content region bordered.')
                }
              ],
            },
          ],
        },
        {
          tag: 'div',
          children: [
            {
              tag: 'input',
              attributes: {
                id: 'multiSelectCheckbox',
                style: 'margin-right:1rem',
                type: 'checkbox'
              }
            },
            {
              tag: 'label',
              attributes: {
                for: 'multiSelectCheckbox'
              },
              children: [
                {
                  text: t('Allow for multiselect (multiple tabs open at once).')
                }
              ],
            },
          ],
        },
        {
          tag: 'div',
          children: [
            {
              tag: 'input',
              attributes: {
                id: 'startCollapsedCheckbox',
                style: 'margin-right:1rem',
                type: 'checkbox'
              }
            },
            {
              tag: 'label',
              attributes: {
                for: 'startCollapsedCheckbox'
              },
              children: [
                {
                  text: t('Start collapsed.')
                }
              ],
            },
          ],
        }
      ],
    });

    // Internal state and configuration for the dialog
    this._internal = {
      id: 'accordionDialog',
      title: t('USWDS Accordion'),
      content: content,
      actionButtons: [
        {
          label: t('OK'),
          withText: true,
          type: 'button',
          onExecute: () => {
            this._saveFormState();
            editor.editing.view.focus();
            this.close();
          }
        },
        {
          label: t('Cancel'),
          withText: true,
          type: 'button',
          onExecute: () => {
            editor.editing.view.focus();
            this.close();
          }
        }
      ],
      onShow: () => this._loadFormState()
    };
  }
}

export default AccordionPropertiesDialog;
